########################################################################################################
#												       #
# support_gwa script.										       # 
# based on the first parameter pased to the script the following functions called:		       #
# 	       	     	       	     	    	       		 	   			       #		       
#	1. prep_data  - to make PC co-variate file						       #
# 	2. qq.plot   - to build QQ plot						       		       #
#	3. mp.plot   - to build MP								       # 
########################################################################################################
args=commandArgs(TRUE)


#########################################################################################################
prep_data = function(plink,pheno,nBase,LOC,PC){								#
#  5 parameters expected:										#
#  	      Genotype file name base (PLINK formatted in  map/ped/raw form) no extension		#
#	      Phenotype file name (Plink formatted: FID, IID, p1.... space or tab separeted)		#
#             Name base to use										#
#	      Location for output files									#
#             number of PC components for the covariate file (Maximum 4)	   			#
#	      	     	   	      	      		     	      					#
#########################################################################################################
	#get a number of columns to expect in ped file
	  ng=as.numeric(strsplit(system(paste("wc -l ",plink,".map",sep=""),intern=T)," ")[[1]][1])*2+6

	  alm=read.table(pheno,header=T)
	  colnames(alm) <- NULL 
	  alm_new=as.matrix(alm)

	  #new map file size (number of raws)
	  ngv=as.numeric(strsplit(system(paste("wc -l ",plink,".map",sep=""),intern=T)," ")[[1]][1])
	  index=as.numeric(strsplit(system(paste("wc -l ",pheno,sep=""),intern=T)," ")[[1]][1])-1
	  ng=ngv+6
	  sn=index

	  # do PCA using all genotypes that have no missing data
	  geno_tmp=scan(paste(plink,".raw",sep=""),what="character")
	  genoFin=matrix(geno_tmp,nrow=ng)

	  genoFin=t(genoFin)
	  genoFin=genoFin[-1,-1:-6]

	  #clear the memory
	  rm(geno_tmp)
	  gc()

	  # missing data filter in here
	  geno_m=matrix(0,nrow=sn,ncol=ngv)
	  count=1
	  for(i in 1:ncol(genoFin)){
  	  	genon=as.numeric(genoFin[,i])
  		miss=sum(is.na(genon))
  		if(miss==0){
			geno_m[,count]=genon
    			count=count+1
  			}
		}

	cat("For PCA ",count, " number of SNPs were used")
	geno_m=geno_m[,1:(count-1)]
	geno_ctr=matrix(0,nrow=sn,ncol=count-1)

	for(i in 1:(count-1)){
  	      geno_ctr[,i]=scale(geno_m[,i])
	      }

	if (any(!is.finite(geno_ctr))) eg=svd(geno_m) else eg=svd(geno_ctr)
	pc=eg$u
	d=eg$d            
	#pdf(paste("eigen_spectrum_",nBase,".pdf",sep=""))
	#plot(d^2/sum(d^2),xlab="Index",ylab="Eigen-Values")
	#dev.off()

	#clear the memory
	rm(genoFin)
	rm(geno_ctr)
	rm(geno_m)
	gc()

	#create a PC covariate file
	if (PC==1) write.table(cbind(alm_new[,1:2],"1",pc[,1]),paste(LOC,"pc_cov1_",nBase,".txt",sep=""),row.names=F,col.names=F,sep="\t",quote=F)
	if (PC==2) write.table(cbind(alm_new[,1:2],"1",pc[,1],pc[,2]),paste(LOC,"pc_cov2_",nBase,".txt",sep=""),row.names=F,col.names=F,sep="\t",quote=F)
	if (PC==3) write.table(cbind(alm_new[,1:2],"1",pc[,1],pc[,2],pc[,3]),paste(LOC,"pc_cov3_",nBase,".txt",sep=""),row.names=F,col.names=F,sep="\t",quote=F)
	if (PC>3) write.table(cbind(alm_new[,1:2],"1",pc[,1],pc[,2],pc[,3],pc[,4]),paste(LOC,"pc_cov4_",nBase,".txt",sep=""),row.names=F,col.names=F,sep="\t",quote=F)
	}


#########################################################################################################
qq.plot = function(dFile, pPos, trait, gType, mType, lambdaOn, serrON,  LOC){				#
	# expected parameters:										#	
#	  1. Datafile name										#
#	  2. P-val column										#
#	  3. Phenotype											#
#	  4. Genotype type										#
#         5. Matrix type										#
#         6. 1/0 plot lambda/not									#
#	  7  1/0 plot standard errors/not								#
#	  8  Location for the plots									#
#########################################################################################################

	  showNames=FALSE
	  pch=NULL
	  col="black"
	  #install.packages("multicore") #IN CASE IF NOT INSTALLED
	  library(multicore)
	  library(GenABEL) #for estlambda function

	  if(as.numeric(lambdaOn)==1){ lambda=TRUE }
	  else { lambda=FALSE } 
	  
	  if(as.numeric(serrON)==1){ errors=TRUE }
	  else { errors=FALSE }

	  data=read.table(dFile, header=F)
	
	  data=as.matrix(data)
	  dimnames(data)=NULL
	  n=nrow(data)

	  p=as.numeric(data[,as.numeric(pPos)])
	  ind_pos=which(p>0)
	  p=p[ind_pos]
	  n=length(p)

	  p_values=p
	  fName=paste(LOC, "QQ_", trait, "_", gType, "_", mType, ".png",sep="")
	  png(fName,width=800,height=600)
	  if( ! is.list(p_values)){

              p_values = as.matrix(p_values)
              p_values_list = list()

              for(i in 1:ncol(p_values) ){
              	    p_values_list[[i]] = p_values[,i]
        	    }

              p_values = p_values_list
              rm( p_values_list )
    	      }

    	  # assign names to list entries if they don't exist
    	  if( is.null( names( p_values ) ) ){
              names( p_values ) = 1:length( p_values )
    	      }

    	  # set pch values if not defined
    	  if( is.null( pch ) ){
              pch = rep(20, length(p_values))
    	      }
	      
    	  if( length(pch) == 1){
              pch = rep(pch, length(p_values))
    	      }

    	  # Set the x and y ranges of the plot to the largest such values
    	  #    encountered in the data
    	  ry = 0; rx = 0

    	  for( key in names( p_values ) ){

               # remove NA values
               p_values[[key]] = p_values[[key]][which( ! is.na( p_values[[key]] ))]
               ry = max(ry, -log10(range(p_values[[key]], finite=TRUE)[1]))
               rx = max(rx, -log10( 1/(length(p_values[[key]])+1)  ))
    	       }

    	  r = max(rx, ry)

    # make initial plot with proper ranges
    plot(1, type='n', las=1, pch=20, xlim=c(0, rx), ylim=c(0, ry), xlab="-log10(expected)", ylab="-log10(observed)", main=paste("QQ plot for ", trait, " phenotype and", gType, "genotype"))
    abline(0, 1)

    lambda_values = c()
    se = c()

    # Plots points for each p-value set
    # Since 90% of p-values should be < .1 and 99% < 0.01, plotting all of these points is
    # time consuming and takes up space, even though all the points appear on top of each other
    # Therefore, thin the p-values at the beginning and increases density for smaller p-values
    # This allows saving QQ plots as a PDF.....with out thinning, a PDF can be VERY large
    i = 1
    for( key in names( p_values ) ){

        observed = sort(p_values[[key]], decreasing=TRUE)
        expected = (length(observed):1) / (length(observed)+1)
        p = length(expected)

        # Thin p-values near 1, and increase density for smaller p-values
        intervals = ceiling(c(1, 0.2*p, 0.4*p, 0.7*p, 0.9*p, 0.95*p, 0.99*p, p))
        scaling = c(28, 200, 800, 1000, 3000, 5000, p)

        for( j in 1:length(scaling) ){
            k = seq(intervals[j], intervals[j+1], intervals[j+1]/scaling[j])
            points(-log10(expected[k]), -log10(observed[k]), col=col[i], pch=pch[i])
        }
        i = i + 1
    }

    # Plot lambda values, if desired
    if( lambda ){
        #result = mclapply( p_values, estlambda, plot=FALSE,method='median')
	result = mclapply( p_values, estlambda, plot=FALSE,method='regression')
        result = matrix(unlist(result), ncol=2, byrow=TRUE)
        lambda_values = result[,1]
        se = result[,2]

        if( ! showNames ){
            namesStrings = rep('', length(names( p_values )) )
        }else{
            namesStrings = paste( names( p_values ), ": ", sep='')
        }

        legend("topleft", legend = paste(namesStrings, round(lambda_values, 3)), col=col, pch=pch, title=expression(lambda[GC]) )
    }

    # Standard errors
    # From Supplementary information from Listgarten et al. 2010. PNAS
    if( errors ){
        error_quantile = 0.95

        M = sum(length(p_values[[key]]))
        alpha = M:1
        beta = M - alpha + 1

        x_top = qbeta(error_quantile, alpha, beta)
        x_bot = qbeta(1-error_quantile, alpha, beta)

        lines( -log10(alpha/(M+1)), -log10(x_top))
        lines( -log10(alpha/(M+1)), -log10(x_bot))
    }
     # Label the bottom of the plot with the file name and current date/time
     mtext(fName, cex=0.75, line=-0.8, side=1, outer=TRUE)
     mtext(format(Sys.time(), "%Y-%m-%d %H:%M"),
        cex=0.75, line=-0.8, side=1, adj=0.01, outer=TRUE)

    dev.off()
    }


#################################################################################################################
mp.plot = function(dFile, nData, axFile, chrNum, fType, subPop, Title, GT, excl, sub, FDR_LIM, COV, LOC){
#  parameters expected:												#
#  	   1.   dataFile name 											#
#	   2.   number of records										#
#	   3.   axisFile name											#
#	   4.   number of chromosomes										#
#	   5. 	trait  	  											# 
#	   6.	sub Pop ( or 0 if none)										#
#	   7.	plot file name to be save as									#
#	   8.	Genotype											#
#	   9. 	number of last chromosomes to be excluded 							#
#	   10.  subject												#
#	   11.  FDR Limit											#
#	   12.  Co-variate.											#
#          13.  Plots location											#
#################################################################################################################
	colors=c("grey","blue","grey","brue")
 
	axis_data=read.table(axFile,header=FALSE)
	chr=as.character(axis_data[1:chrNum,1])
	 
	brk=axis_data[,2]
	axis=rep(0,chrNum)
	for(i in 1:chrNum){
	   	 axis[i]=sum(as.numeric(brk[i:(i+1)]))/2
		 }

	vect <- scan(dFile,what="double")
	data<-t(matrix(vect,3,as.numeric(nData)))
	dimnames(data)=NULL

	id=as.character(data[,1])
	pos=data[,2]
	pos=as.numeric(pos)
	p=data[,3]
	p_num=as.double(as.character(p))
	fdr=p.adjust(p_num, method ="BH")
	minFDR=min(fdr)  #min FDR to place in the notes in case if threshold is not met
	maxFDR=0
	nextFDR=0
	maxFDR=max(fdr[fdr<=FDR_LIM])   ##maximum FDR below the threshold
	nextFDR=min(fdr[fdr>FDR_LIM])
	log_p=-log(p_num,base=10)

	if (!(file.exists(paste(LOC,"limits_FDR.txt",sep="")))) {
    	   write(paste("Title","FDR_LIMIT","FDR_MIN","pVal@FDR_LIMIT",sep="\t"),file="limits_FDR.txt",append=FALSE)
	   }
	png(paste(LOC,"MP_",Title,".png",sep=""),width=800,height=600)

	par(mar=c(6,4,4,2))
	plot(x="",y="",ylim=c(0,max(log_p,na.rm=T)),xlim=c(0,max(pos)),xlab="",ylab="",axes=F)
	axis(1,at=axis[1:length(axis)-excl],tick=F,label=chr[1:chrNum-excl],lwd=2.5,font.axis=1,cex.axis=2,las=3,line=-0.8)
	axis(2,at=axTicks(2),lwd=2.5,font.axis=1,cex.axis=2,line=-2)
	pFDR=0
	# add FDR threshold if met
	if(maxFDR>0){
		maxLP=min(log_p[fdr==maxFDR])
		minMaxLP=min(maxLP)
	
		nextLP=max(log_p[fdr==nextFDR])
		mNextLP=max(nextLP)

		#fdrR=(nextFDR-FDR_LIM)/(nextFDR-maxFDR)
		fdrR=(log(nextFDR,base=10)-log(FDR_LIM,base=10))/(log(nextFDR,base=10)-log(maxFDR,base=10))
		pFDR=mNextLP+fdrR*(minMaxLP-mNextLP)
	
		abline(h=pFDR,lty=4,untf=FALSE)
		text(100, pFDR+0.3, paste("P ",round(pFDR,1)," for FDR limit ",round(FDR_LIM,2),sep=""), cex=0.85, pos=4, col="red",offset=38)
		}

	mtext(side=1,text="Chromosome",font=1,cex=2,line=3.5)
	mtext(side=2,text=expression(paste(-Log[10],"(",italic(P),")",sep="")),font=1,cex=2,line=1)


	bon=10000
	colInd=1
	color="blue"
	for(j in seq(1,chrNum,by=1)){
      	      points(pos[id==chr[j] & log_p<=bon],log_p[id==chr[j] & log_p<=bon],pch=20,col=color,cex=1)
      	      if(colInd == 4) colInd=1 else  colInd = colInd + 1
      	      if (color=="grey") color="blue" else color="grey"
	      }

	if (subPop=='0') SUBB = "" else SUBB = paste(subPop," sub-pop.",sep="")

	title(main = list(paste(sub,". ",fType," phenotype.",GT," genotype.",COV," co-variate.",SUBB,sep=""), cex=1.5,col="red", font=3))

	# Label the bottom of the plot with the file name and current date/time
  	mtext(paste("MP_",Title,".png    Minimum FDR: ",round(minFDR,3),".", sep=""), cex=0.8, line=-0.8, side=1, outer=TRUE)
  	mtext(format(Sys.time(), "%Y-%m-%d %H:%M"),
        cex=0.8, line=-0.8, side=1, adj=0.01, outer=TRUE)
	write(paste(Title,FDR_LIM,round(minFDR,4),round(pFDR,2),sep="\t"),file="limits_FDR.txt",append=TRUE)
	dev.off()
	}


#########################################################################################################################
#															#
# main execution. based on the first parameter pased to the script the following functions called:			#
#      		  	       	     	       	     	    	       		 	   				# 
#	1. prep_data call												#
# 	2. qq.plot   													#
#	3. mp.plot 													#
#########################################################################################################################

if (args[1]=='1'){
   prep_data(args[2], args[3], args[4], args[5], as.numeric(args[6]))
   }

if (args[1]=='2'){
   qq.plot(args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9])
   }
  
if (args[1]=='3'){
   mp.plot(args[2], args[3], args[4], as.numeric(args[5]), args[6], args[7], args[8], args[9], as.numeric(args[10]), args[11], as.numeric(args[12]), args[13], args[14])
   }
